'use strict';

var React = require('react');
var d3 = require('d3');
var common = require('./common');
var Chart = common.Chart;
var XAxis = common.XAxis;
var YAxis = common.YAxis;

var Area = React.createClass({displayName: "Area",

  propTypes: {
    path: React.PropTypes.string,
    fill: React.PropTypes.string
  },

  getDefaultProps: function() {
    return {
      fill: '#3182bd'
    };
  },

  render: function() {

    return (
      React.createElement("path", {
        className: "area-path", 
        d: this.props.path, 
        fill: this.props.fill}
      )
    );
  }

});


var DataSeries = exports.DataSeries = React.createClass({displayName: "DataSeries",

  render:function() {

    var props = this.props;

    var area = d3.svg.area()
      .x(function(d) { return props.xScale(props.xAccessor(d)); })
      .y0(function(d) { return props.yScale(d.y0); })
      .y1(function(d) { return props.yScale(d.y0 + props.yAccessor(d)); });

    var path = area(props.data);

    return (
      React.createElement(Area, {fill: props.colors(props.name), path: path})
    );
  }

});


var AreaChart = exports.AreaChart = React.createClass({displayName: "AreaChart",

  propTypes: {
    data: React.PropTypes.oneOfType([
      React.PropTypes.array,
      React.PropTypes.object
    ]),
    legend: React.PropTypes.bool,
    legendOffset: React.PropTypes.number,
    yAxisTickCount: React.PropTypes.number,
    xAxisTickInterval: React.PropTypes.object,
    colors: React.PropTypes.func,
    width: React.PropTypes.number,
    height: React.PropTypes.number,
    title: React.PropTypes.string,
    xAccessor: React.PropTypes.func,
    yAccessor: React.PropTypes.func
  },

  getDefaultProps:function() {
    return {
      data: [],
      colors: d3.scale.category20c(),
      margins: {top: 10, right: 20, bottom: 30, left: 30},
      legendOffset: 120,
      legend: false,
      yAxisTickCount: 4,
      width: 400,
      height: 200,
      title: '',
      xAccessor: function(d)  {return d.x;},
      yAccessor: function(d)  {return d.y;}
    };
  },

  render:function() {

    var props = this.props;

    // Calculate inner chart dimensions
    var chartWidth, chartHeight;
    chartWidth = props.width - props.margins.left - props.margins.right;
    chartHeight = props.height - props.margins.top - props.margins.bottom;

    if (props.legend) {
      chartWidth = chartWidth - props.legendOffset;
    }

    if (!Array.isArray(props.data)) {
      props.data = [props.data];
    }

    var yScale = d3.scale.linear()
      .range([chartHeight, 0]);

    var xValues = [];
    var yValues = [];
    var seriesNames = [];
    props.data.forEach( function(series)  {
      seriesNames.push(series.name);
      series.values.forEach(function(val, idx)  {
        xValues.push(props.xAccessor(val));
        yValues.push(props.yAccessor(val));
      });
    });

    var xScale;
    if (xValues.length > 0 && Object.prototype.toString.call(xValues[0]) === '[object Date]' && props.xAxisTickInterval) {
      xScale = d3.time.scale()
        .range([0, chartWidth]);
    } else {
      xScale = d3.scale.linear()
        .range([0, chartWidth]);
    }

    xScale.domain(d3.extent(xValues));
    yScale.domain(d3.extent(yValues));

    props.colors.domain(seriesNames);

    var stack = d3.layout.stack()
      .x(props.xAccessor)
      .y(props.yAccessor)
      .offset('expand')
      .order('reverse')
      .values(function(d) { return d.values; });

    var layers = stack(props.data);

    var trans = "translate(" + props.margins.left + "," + props.margins.top + ")";

    var dataSeries = layers.map( function(d, idx)  {
      return (
          React.createElement(DataSeries, {
            key: idx, 
            name: d.name, 
            colors: props.colors, 
            index: idx, 
            xScale: xScale, 
            yScale: yScale, 
            data: d.values, 
            xAccessor: props.xAccessor, 
            yAccessor: props.yAccessor}
          )
        );
      });

    return (
      React.createElement(Chart, {
        legend: props.legend, 
        data: props.data, 
        margins: props.margins, 
        colors: props.colors, 
        width: props.width, 
        height: props.height, 
        title: props.title
      }, 
        React.createElement("g", {transform: trans}, 
          dataSeries, 
          React.createElement(XAxis, {
            xAxisClassName: "area x axis", 
            xScale: xScale, 
            xAxisTickInterval: props.xAxisTickInterval, 
            xAxisTickCount: props.xAxisTickCount, 
            margins: props.margins, 
            width: chartWidth, 
            height: chartHeight}
          ), 
          React.createElement(YAxis, {
            yAxisClassName: "area y axis", 
            yScale: yScale, 
            margins: props.margins, 
            yAxisTickInterval: props.yAxisTickInterval, 
            yAxisTickCount: this.props.yAxisTickCount, 
            width: chartWidth, 
            height: props.height}
          )
        )
      )
    );
  }

});
